==> directord-0.11.3a20211218144122.tar.gz: directord-0.11.3a20211218144122/directord/interface.py <==
#   Copyright Peznauts <kevin@cloudnull.com>. All Rights Reserved.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.

import decimal
import logging
import os
import time

import directord

from directord import utils


class Interface(directord.Processor):
    """The Interface class.

    This class defines everything required to connect to or from a given
    server.
    """

    uuid = utils.get_uuid()

    def __init__(self, args):
        """Initialize the interface class.

        :param args: Arguments parsed by argparse.
        :type args: Object
        """

        super(Interface, self).__init__()

        self.args = args

        # Set log handlers to debug when enabled.
        if self.args.debug:
            self.log.setLevel(logging.DEBUG)
            for handler in self.log.handlers:
                handler.setLevel(logging.DEBUG)

        try:
            self.heartbeat_interval = self.args.heartbeat_interval
        except AttributeError:
            self.heartbeat_interval = 1

        self.base_dir = "/etc/directord"
        self.public_keys_dir = os.path.join(self.base_dir, "public_keys")
        self.secret_keys_dir = os.path.join(self.base_dir, "private_keys")
        self.keys_exist = os.path.exists(
            self.public_keys_dir
        ) and os.path.exists(self.secret_keys_dir)

        try:
            _driver = directord.plugin_import(
                plugin=".drivers.{}".format(self.args.driver)
            )
        except Exception as e:
            raise SystemExit(
                "Driver was not able to be loaded: {}".format(str(e))
            )
        else:
            self.log.debug(
                "Loading messaging driver: [ %s ]", self.args.driver
            )
            try:
                self.driver = _driver.Driver(
                    args=self.args,
                    encrypted_traffic_data={
                        "enabled": self.keys_exist,
                        "public_keys_dir": self.public_keys_dir,
                        "secret_keys_dir": self.secret_keys_dir,
                    },
                    interface=self,
                )
            except NameError as e:
                raise OSError(
                    "Failed to load driver {} - Error: {} - Check"
                    " configuration and dependency installation.".format(
                        self.args.driver, str(e)
                    )
                ) from None


class Worker:
    """Worker class object."""

    def __init__(self, identity):
        """Initialize the worker object."""

        self.identity = identity
        self.expire_time = None
        self.machine_id = None
        self.version = None
        self.host_uptime = None
        self.agent_uptime = None
        self.version = None
        self.driver = None

    @property
    def expired(self):
        """Return Boolean, True if expiry is greater than Now or None."""

        if self.expiry is None:
            return True
        else:
            return time.time() >= self.expire_time

    @property
    def expiry(self):
        """Return Float, for expiry."""

        return self.expire_time - time.time()


class Job:
    """Job class object."""

    def __init__(self, job_item):
        """Initialize the job object."""

        self._createtime = time.time()
        self._executiontime = dict()
        self._lasttime = time.time()
        self._processing = dict()
        self._roundtripltime = dict()

        self.job_id = job_item["job_id"]

        self.JOB_DEFINITION = job_item
        self.JOB_SHA3_224 = job_item["job_sha3_224"]
        self.JOB_NAME = job_item.get("job_name", self.JOB_SHA3_224)
        self.PARENT_JOB_ID = job_item.get("parent_id")
        self.PARENT_JOB_NAME = job_item.get("parent_name", self.PARENT_JOB_ID)
        self.VERB = job_item["verb"]
        self.COMPONENT_TIMESTAMP = None
        self.INFO = dict()
        self.PROCESSING = None
        self.RETURN_TIMESTAMP = None
        self.STDERR = dict()
        self.STDOUT = dict()

    @property
    def failed(self):
        """Return True or Flase if job failed."""

        return len(self.failed_nodes) > 0

    @property
    def _nodes(self):
        """Return a sorted list of all nodes."""

        return sorted(self._processing.keys())

    def _check_nodes(self, status_code):
        """Return a list of nodes based on a defined status code.

        :param status_code: Status code string.
        :type status_code: String
        :returns: List
        """
        nodes = list()
        for k, v in self._processing.items():
            if v == status_code:
                nodes.append(k)
        return nodes

    @property
    def failed_nodes(self):
        """Return a list of failed nodes."""

        return self._check_nodes(status_code="\x15")

    @property
    def success_nodes(self):
        """Return a list of success nodes."""

        return self._check_nodes(status_code="\x04")

    @property
    def processing(self):
        """Set the processing flag and return boolean if processing."""

        processing = any(self._check_nodes(status_code="\x16"))
        if processing:
            self.PROCESSING = "\x16"
        else:
            self.PROCESSING = "\x04"
        return processing

    def set_roundtripltime(self, identity, recv_time):
        """Set the round trip time.

        The constante ROUNDTRIP_TIME is used to represent the average
        roundtrip time.
        """

        if isinstance(recv_time, (int, float)):
            self._roundtripltime[identity] = (
                float(recv_time) - self._createtime
            )

        try:
            self.ROUNDTRIP_TIME = "{:.8f}".format(
                decimal.Decimal(
                    sum(self._roundtripltime.values())
                    / len(self._roundtripltime.keys())
                )
            )
        except ZeroDivisionError:
            pass

    def set_executiontime(self, identity, execution_time):
        """Set the execution time.

        The constante EXECUTION_TIME is used to represent the average
        execution time.
        """

        if isinstance(execution_time, (int, float)):
            self._executiontime[identity] = float(execution_time)

        try:
            self.EXECUTION_TIME = "{:.8f}".format(
                decimal.Decimal(
                    sum(self._executiontime.values())
                    / len(self._executiontime.keys())
                )
            )
        except ZeroDivisionError:
            pass

==> directord-0.12.0a20220106175318.tar.gz: directord-0.12.0a20220106175318/directord/interface.py <==
#   Copyright Peznauts <kevin@cloudnull.com>. All Rights Reserved.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.

import logging
import os
import signal

import directord

from directord import utils


class Interface(directord.Processor):
    """The Interface class.

    This class defines everything required to connect to or from a given
    server.
    """

    uuid = utils.get_uuid()

    def __init__(self, args):
        """Initialize the interface class.

        :param args: Arguments parsed by argparse.
        :type args: Object
        """

        super(Interface, self).__init__()

        self.args = args

        # Set log handlers to debug when enabled.
        if self.args.debug:
            self.log.setLevel(logging.DEBUG)
            for handler in self.log.handlers:
                handler.setLevel(logging.DEBUG)

        try:
            self.heartbeat_interval = self.args.heartbeat_interval
        except AttributeError:
            self.heartbeat_interval = 1

        self.base_dir = "/etc/directord"
        self.public_keys_dir = os.path.join(self.base_dir, "public_keys")
        self.secret_keys_dir = os.path.join(self.base_dir, "private_keys")
        self.keys_exist = os.path.exists(
            self.public_keys_dir
        ) and os.path.exists(self.secret_keys_dir)
        try:
            self.driver = self._load_driver(driver=self.args.driver)
        except AttributeError as e:
            self.log.warning(
                "Falling back with dummy driver due error [ %s ] in driver"
                " [ %s ]. Check the driver CLI arguments, the configuration"
                " file [ %s ] contents, and ensure all dependencies are"
                " installed.",
                str(e),
                self.args.driver,
                self.args.config_file,
            )
            self.driver = self._load_driver(driver="dummy")

    def _load_driver(self, driver):
        try:
            _driver = directord.plugin_import(
                plugin=".drivers.{}".format(driver)
            )
        except Exception as e:
            raise SystemExit(
                "Driver was not able to be loaded: {}".format(str(e))
            )
        else:
            self.log.debug("Loading messaging driver: [ %s ]", driver)
            try:
                return _driver.Driver(
                    args=self.args,
                    encrypted_traffic_data={
                        "enabled": self.keys_exist,
                        "public_keys_dir": self.public_keys_dir,
                        "secret_keys_dir": self.secret_keys_dir,
                    },
                    interface=self,
                )
            except NameError as e:
                raise OSError(
                    "Failed to load driver {} - Error: {} - Check"
                    " configuration and dependency installation.".format(
                        driver, str(e)
                    )
                ) from None


class ProcessInterface(Interface):
    """The ProcessInterface class.

    This class defines everything required to execute the application process.
    """

    def __init__(self, args):
        """Initialize the process interface class.

        :param args: Arguments parsed by argparse.
        :type args: Object
        """

        super(ProcessInterface, self).__init__(args=args)
        signal.signal(signal.SIGINT, self.exit_gracefully)
        signal.signal(signal.SIGTERM, self.exit_gracefully)

    def exit_gracefully(self, *args, **kwargs):
        """Handle a graceful exit of the application."""

        raise SystemExit("Stop signal intercepted {}".format(args))
